from openlineage.lineage_backend import Backend
import logging
import traceback
from airflow.lineage.backend import LineageBackend
from alvin_airflow import utils


# TODO Refactor the Alvin API Key injection
###########################################
API_KEY_HEADER = "X-API-KEY"


def _add_auth_aws_flavor(self, api_key: str):
    self.session.headers.update({API_KEY_HEADER: api_key})
    print(
        f"Alvin patched the OL API KEY header as {API_KEY_HEADER}={self.session.headers.get(API_KEY_HEADER)}"
    )


def patch_open_lineage_api_key_header():
    from openlineage.client import OpenLineageClient

    OpenLineageClient._add_auth_old = OpenLineageClient._add_auth

    OpenLineageClient._add_auth = _add_auth_aws_flavor


patch_open_lineage_api_key_header()
############################################


class AlvinBackend(Backend):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class AlvinLineageBackend(LineageBackend):
    # Airflow 1.10 uses send_lineage as staticmethod, so just construct class
    # instance on first use and delegate calls to it
    backend: AlvinBackend = None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        utils.set_global_vars()

    @classmethod
    def send_lineage(cls, *args, **kwargs):
        """Alvin implementation of the send_lineage backend."""
        print("calling send lineage with args and kwargs")
        if not cls.backend:
            print("Lineage backend is none, creating it")
            cls.backend = AlvinBackend()
        return cls.backend.send_lineage(*args, **kwargs)


log = logging.getLogger(__name__)


def alvin_callback(context, operator, is_airflow_legacy=False):
    """Alvin callback implementation.

    This function use the OpenLineage implementation of
    the Airflow Backend and call send_lineage with the
    given context.
    """
    try:
        if is_airflow_legacy:
            AlvinLineageBackend.send_lineage(context=context, operator=operator)  # noqa
        else:
            lineage_backend = AlvinLineageBackend()
            lineage_backend.send_lineage(context=context, operator=operator)
    except Exception:
        log.error(f"Alvin Callback Error: {traceback.format_exc()}")
