import logging

from openlineage.airflow.extractors.base import TaskMetadata
from openlineage.airflow.extractors.snowflake_extractor import SnowflakeExtractor

from alvin_airflow import utils

log = logging.getLogger(__name__)


class AlvinSFExtractor(SnowflakeExtractor):
    def __init__(self, operator):
        print("Init in AlvinSFExtractor")
        super().__init__(operator)
        utils.print_alvin_environment()

    # make all versions work
    def _get_hook(self):
        print("_get_hook in AlvinSFExtractor")
        if hasattr(self.operator, "get_db_hook"):
            # Airflow v2
            return self.operator.get_db_hook()
        # Airflow v1
        return self.operator.get_hook()

    # change from
    # openlineage/airflow/extractors/snowflake_extractor.py
    # line 35
    def _get_database(self) -> str:
        print("_get_database in AlvinSFExtractor")
        log.warning(f"OVERRIDING OPENLINEAGE FOR SNOWFLAKE: {str(self._get_hook())}")
        # return self.operator.get_db_hook()._get_conn_params()['database']
        return self._get_hook().database

    # change from
    # openlineage/airflow/extractors/snowflake_extractor.py
    # line 38
    def _get_authority(self) -> str:
        print("_get_authority in AlvinSFExtractor")
        log.warning(f"OVERRIDING OPENLINEAGE FOR SNOWFLAKE: {str(self._get_hook())}")
        # return self.operator.get_db_hook()._get_conn_params()['account']
        return self._get_hook().snowflake_conn_id

    def _get_query_ids(self):
        print("_get_query_ids in AlvinSFExtractor")
        if hasattr(self.operator, "query_ids"):
            # Airflow v2
            return self.operator.query_ids
        # Airflow v1
        return self.operator.sfqids

    # def extract(self, task_instance) -> TaskMetadata:
    #     print("extract in AlvinSFExtractor")
    #     return self.extract_on_complete(task_instance)

    # [2022-03-11 09:27:23,224] {__init__.py:93} ERROR - Failed to extract metadata extract() missing 1 required positional argument: 'task_instance' task_type=SnowflakeOperator airflow_dag_id=snowflake_dag_airflow_v2 task_id=task_sf_af_v2 airflow_run_id=None
    # Traceback (most recent call last):
    #   File "/home/airflow/.local/lib/python3.8/site-packages/openlineage/lineage_backend/__init__.py", line 85, in _extract_metadata
    #     task_metadata = self._extract(extractor, task_instance)
    #   File "/home/airflow/.local/lib/python3.8/site-packages/openlineage/lineage_backend/__init__.py", line 106, in _extract
    #     task_metadata = extractor.extract_on_complete(task_instance)
    #   File "/opt/airflow/alvin_airflow/alvin_airflow/snowflake.py", line 58, in extract_on_complete
    #     tm = super().extract_on_complete(task_instance)
    #   File "/home/airflow/.local/lib/python3.8/site-packages/openlineage/airflow/extractors/base.py", line 63, in extract_on_complete
    #     return self.extract()
    # TypeError: extract() missing 1 required positional argument: 'task_instance'
    def extract_on_complete(self, task_instance) -> TaskMetadata:
        print("extract_on_complete in AlvinSFExtractor")
        tm = super().extract_on_complete(task_instance)

        # TODO sort out Job Facet
        log.warning("Alvin SNOWFLAKE TODO sort out Job Facet")
        db_hook = self._get_hook()
        conn_params = db_hook._get_conn_params()
        log.warning(f"Alvin SNOWFLAKE ON COMPLETE - conn params: {conn_params}")
        query_ids = self._get_query_ids()
        log.warning(f"Alvin SF query IDs on extract on complete? {query_ids}")
        tm.job_facets["alvin"] = {
            "snowflake_query_ids": query_ids,
        }
        return tm
