from airflow.models import DAG
from airflow.operators.python import PythonOperator
from datetime import datetime, timedelta
import requests
from airflow.utils.session import provide_session
from alvin_airflow.config import ALVIN_BACKEND_API_URL, ALVIN_BACKEND_API_KEY


seven_days_ago = datetime.combine(
    datetime.today() - timedelta(1), datetime.min.time()
)  # noqa

args = {
    "owner": "airflow",
    "start_date": seven_days_ago,
}


@provide_session
def extract_metadata(session):
    """Extract metadata from DAG and send to Alvin Backend API."""
    from airflow.models.serialized_dag import SerializedDagModel
    from airflow.serialization.serialized_objects import SerializedDAG

    dag_metadata = SerializedDagModel.read_all_dags()

    alvin_backend_metadata_url = f'{ALVIN_BACKEND_API_URL}/api/metadata'

    for dag_id, serialized_dag in dag_metadata.items():
        row = SerializedDagModel.get(dag_id, session)
        dag = row.dag
        dag_metadata = SerializedDAG.to_dict(dag)
        payload = {'dag_ig': dag_id, 'serialized_dag': dag_metadata['dag']}
        requests.post(alvin_backend_metadata_url, json=payload, headers={"X-API-KEY": ALVIN_BACKEND_API_KEY})
        print(f'Metadata extracted: {payload}')


alvin_metadata_extractor = DAG(
    dag_id="alvin_metadata_extractor", default_args=args, schedule_interval="@hourly", catchup=False
)  # noqa

task_success_one = PythonOperator(
    task_id="dag_metadata_extractor",
    provide_context=True,
    python_callable=extract_metadata,
    dag=alvin_metadata_extractor,
)
